@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2022 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -426,6 +426,15 @@ class _TestNullAuth(ServerTestCase):
 
         return await cls.create_server(_NullServer)
 
+    @asynctest
+    async def test_get_server_auth_methods(self):
+        """Test getting auth methods from the test server"""
+
+        auth_methods = await asyncssh.get_server_auth_methods(
+            self._server_addr, self._server_port)
+
+        self.assertEqual(auth_methods, ['none'])
+
     @asynctest
     async def test_disabled_auth(self):
         """Test disabled authentication"""
@@ -452,6 +461,15 @@ class _TestGSSAuth(ServerTestCase):
 
         return await cls.create_server(_AsyncGSSServer, gss_host='1')
 
+    @asynctest
+    async def test_get_server_auth_methods(self):
+        """Test getting auth methods from the test server"""
+
+        auth_methods = await asyncssh.get_server_auth_methods(
+            self._server_addr, self._server_port)
+
+        self.assertEqual(auth_methods, ['gssapi-with-mic'])
+
     @asynctest
     async def test_gss_kex_auth(self):
         """Test GSS key exchange authentication"""
@@ -468,6 +486,22 @@ class _TestGSSAuth(ServerTestCase):
                                 username='user', gss_host='1'):
             pass
 
+    @asynctest
+    async def test_gss_mic_auth_sign_error(self):
+        """Test GSS MIC authentication signing failure"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(kex_algs=['ecdh-sha2-nistp256'],
+                               username='user', gss_host='1,sign_error')
+
+    @asynctest
+    async def test_gss_mic_auth_verify_error(self):
+        """Test GSS MIC authentication signature verification failure"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(kex_algs=['ecdh-sha2-nistp256'],
+                               username='user', gss_host='1,verify_error')
+
     @asynctest
     async def test_gss_delegate(self):
         """Test GSS credential delegation"""
@@ -615,6 +649,15 @@ class _TestHostBasedAuth(ServerTestCase):
         return await cls.create_server(
             _HostBasedServer, known_client_hosts='known_hosts')
 
+    @asynctest
+    async def test_get_server_auth_methods(self):
+        """Test getting auth methods from the test server"""
+
+        auth_methods = await asyncssh.get_server_auth_methods(
+            self._server_addr, self._server_port, username='user')
+
+        self.assertEqual(auth_methods, ['hostbased'])
+
     @asynctest
     async def test_client_host_auth(self):
         """Test connecting with host-based authentication"""
@@ -1010,6 +1053,15 @@ class _TestPublicKeyAuth(ServerTestCase):
 
         return conn
 
+    @asynctest
+    async def test_get_server_auth_methods(self):
+        """Test getting auth methods from the test server"""
+
+        auth_methods = await asyncssh.get_server_auth_methods(
+            self._server_addr, self._server_port)
+
+        self.assertEqual(auth_methods, ['publickey'])
+
     @asynctest
     async def test_encrypted_client_key(self):
         """Test public key auth with encrypted client key"""
@@ -1694,6 +1746,15 @@ class _TestPasswordAuth(ServerTestCase):
 
         return await cls.create_server(_PasswordServer)
 
+    @asynctest
+    async def test_get_server_auth_methods(self):
+        """Test getting auth methods from the test server"""
+
+        auth_methods = await asyncssh.get_server_auth_methods(
+            self._server_addr, self._server_port, username='pw')
+
+        self.assertEqual(auth_methods, ['keyboard-interactive', 'password'])
+
     @async_context_manager
     async def _connect_password(self, username, password, old_password='',
                                 new_password='', disable_trivial_auth=False,
@@ -1800,6 +1861,15 @@ class _TestKbdintAuth(ServerTestCase):
 
         return await cls.create_server(_KbdintServer)
 
+    @asynctest
+    async def test_get_server_auth_methods(self):
+        """Test getting auth methods from the test server"""
+
+        auth_methods = await asyncssh.get_server_auth_methods(
+            self._server_addr, self._server_port, username='none')
+
+        self.assertEqual(auth_methods, ['keyboard-interactive'])
+
     @async_context_manager
     async def _connect_kbdint(self, username, responses, test_async=False):
         """Open a connection to test keyboard-interactive auth"""
