@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2015-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -42,7 +42,7 @@ from asyncssh.kex import register_kex_alg, get_kex_algs, get_kex
 from asyncssh.packet import SSHPacket, Boolean, Byte, MPInt, String
 from asyncssh.public_key import decode_ssh_public_key
 
-from .util import asynctest, gss_available, patch_gss
+from .util import asynctest, get_test_key, gss_available, patch_gss
 from .util import AsyncTestCase, ConnectionStub
 
 
@@ -209,7 +209,7 @@ class _KexServerStub(_KexConnectionStub):
         if gss_host and 'no_host_key' in gss_host:
             self._server_host_key = None
         else:
-            priv_key = asyncssh.generate_private_key('ssh-rsa')
+            priv_key = get_test_key('ssh-rsa')
             self._server_host_key = asyncssh.load_keypairs(priv_key)[0]
 
     def connection_lost(self, exc):
@@ -331,7 +331,7 @@ class _TestKex(AsyncTestCase):
         with self.subTest('Invalid signature'):
             with self.assertRaises(asyncssh.KeyExchangeFailed):
                 client_conn.start()
-                client_conn.simulate_dh_reply(host_key.public_data, 1, b'')
+                client_conn.simulate_dh_reply(host_key.public_data, 2, b'')
 
         client_conn.close()
         server_conn.close()
@@ -399,7 +399,8 @@ class _TestKex(AsyncTestCase):
 
         with self.subTest('Signature verification failure'):
             with self.assertRaises(asyncssh.KeyExchangeFailed):
-                await self._check_kex(b'gss-group1-sha1-mech', '0,fail')
+                await self._check_kex(b'gss-group1-sha1-mech',
+                                      '0,verify_error')
 
         with self.subTest('Empty token in init'):
             with self.assertRaises(asyncssh.ProtocolError):
@@ -572,7 +573,7 @@ class _TestKex(AsyncTestCase):
         with self.subTest('Invalid signature'):
             with self.assertRaises(asyncssh.KeyExchangeFailed):
                 host_key = server_conn.get_server_host_key()
-                trans_key = asyncssh.generate_private_key('ssh-rsa', 2048)
+                trans_key = get_test_key('ssh-rsa', 2048)
                 client_conn.simulate_rsa_pubkey(host_key.public_data,
                                                 trans_key.public_data)
                 client_conn.simulate_rsa_done(b'')
