@@ -57,6 +57,7 @@ from asyncssh.gss import gss_available
 from asyncssh.logging import logger
 from asyncssh.misc import ConnectionLost, SignalReceived
 from asyncssh.packet import Byte, String, UInt32, UInt64
+from asyncssh.public_key import generate_private_key
 
 
 # pylint: disable=no-member
@@ -71,6 +72,9 @@ else:
 # pylint: enable=no-member
 
 
+_test_keys = {}
+
+
 def asynctest(coro):
     """Decorator for async tests, for use with AsyncTestCase"""
 
@@ -157,6 +161,20 @@ def _encode_options(options):
     return b''.join((String(k) + String(v) for k, v in options.items()))
 
 
+def get_test_key(alg_name, key_id=0, **kwargs):
+    """Generate or return a key with the requested parameters"""
+
+    params = tuple((alg_name, key_id)) + tuple(kwargs.items())
+
+    try:
+        key = _test_keys[params]
+    except KeyError:
+        key = generate_private_key(alg_name, **kwargs)
+        _test_keys[params] = key
+
+    return key
+
+
 def make_certificate(cert_version, cert_type, key, signing_key, principals,
                      key_id='name', valid_after=0,
                      valid_before=0xffffffffffffffff, options=None,
@@ -192,7 +210,8 @@ def run(cmd):
         return subprocess.check_output(cmd, shell=True,
                                        stderr=subprocess.STDOUT)
     except subprocess.CalledProcessError as exc: # pragma: no cover
-        print(exc.output.decode())
+        logger.error('Error running command: %s' % cmd)
+        logger.error(exc.output.decode())
         raise
 
 
